/**********************************************************************
 * File Name: Log.java
 *
 * Copyright (C) 2000, US Air Force Research Laboratory,
 * Rome Research Site, Rome NY 13424, USA.
 *
 * Use of this software implies acceptance of the terms described in
 * accompanying license.txt file.
 *
 *
 * Authors: James Lawton, Mark MacBeth
 *
 * Creation Date: May 2000
 *
 * Description: 
 *   It's lo-og, lo-og, 
 *   It's big, it's heavy, it's wood. 
 *   It's lo-og, lo-og, 
 *   It's better than bad, it's good. 
 * (Log file generator)
 *
 * Modification History:
 *
 **********************************************************************/

// mwm: Log must have a package in the HW environment, unlike the simulator
// package runtime.util; // Not in Radsim, though -- jhl

import java.lang.*;
import java.io.*;
import java.util.*;

public class Log {

    // Logging level:
    public static final int LOG_NONE = 0;
    public static final int LOG_BASIC = 1;
    public static final int LOG_VERBOSE = 2;
    public static int loglevel = LOG_NONE;

    // Log file name
    public static String default_logfile = "log.txt";

    // Log file output stream
    private static PrintStream logstream = null;


    // Logger initialization.  Must be called before using
    // logger, or log file will not be written.
    public static void init() {
	init(default_logfile, LOG_BASIC);
    }

    public static void init(String logfile, int logLevel) {
	// Open log file stream
	try {
	    logstream = new PrintStream(new FileOutputStream(logfile));
	}
	catch (Exception e) { 
	    e.printStackTrace(); 
	}

	loglevel = logLevel;
    }

    // Shut down the logger
    public static void shutdown () {
	if (logstream != null)
	    logstream.close();
    }


    // Basic logging method.  Just write a string to the logfile
    // if the LOGSTEAM has been established and the logging level
    // is at least BASIC.  More specific methods should check the
    // logging level for further filtering.
    public static synchronized void log(String logstring, int verbosity) {
	if (logstream != null && verbosity <= loglevel)
	    logstream.println(logstring);
    }

    public static void logBasic(String logstring) {
	log(logstring, LOG_BASIC);
    }

    public static void logVerbose(String logstring) {
	log(logstring, LOG_VERBOSE);
    }


    // General state information
    public static void roomsize(int width, int height) {
	logBasic("(roomsize " + width + " " + height + ")");
    }

    public static void timestamp(int simtime) {
	logBasic("(timestamp " + simtime + ")");
    }

    // General sensor state information
    public static void defSensor(String name, int simtime, double x, double y, 
				 double orientation, int recChannel, int transChannel,
				 int duration, int size, int gain, int mode, 
				 int currentSector, int activeSectors[]) {

	       String infoString = 
		   "(defsensor " + name + " " + simtime + 
		   "\n    location " + x + " " + y + " 0.0" +
		   "\n    orientation " + orientation +
		   "\n    receiveChannel " + recChannel +
		   "\n    transmitChannel " + transChannel +
		   "\n    dataCollectionMode " + mode +
		   "\n    measurementDuration " + duration +
		   "\n    sampleSize " + size +
		   "\n    gain " + gain +
		   "\n    currentSector " + currentSector +
		   "\n    activeSectors";

	       for (int sector = 0; sector < activeSectors.length; sector++) {
		   if (activeSectors[sector] >= 0)
		       infoString += " " + sector;
	       }

	       infoString += ")";

	       logBasic(infoString);
    }

    public static void sensorLocation(String name, int simtime, double x, double y) {
	logBasic("(sensor " + name + " " + simtime + 
		 " location " + x + " " + y + " 0.0)");
    }

    public static void sensorOrientation(String name, int simtime, double orientation) {
	logBasic("(sensor " + name + " " + simtime + 
		 " orientation " + orientation + ")");
    }


    // State changes that correspond to Node methods:
    public static void receiverActivate(String name, int simtime) {
	logBasic("(sensor " + name + " " + simtime + 
		 " receiverActivate)");
    }

    public static void receiverDeactivate(String name, int simtime) {
	logBasic("(sensor " + name + " " + simtime +
		 " receiverDeactivate)");
    }

    public static void receiverSetChannel(String name, int simtime, int channel) {
	logBasic("(sensor " + name + " " + simtime +
		 " receiverSetChannel " + channel + ")");
    }

    public static void transmitterSetChannel(String name, int simtime, int channel) {
	logBasic("(sensor " + name + " " + simtime +
		 " transmitterSetChannel " + channel + ")");
    }

    public static void messageReceived(String name, int simtime, String src, int length) {
	logBasic("(sensor " + name + " " + simtime +
		 " messageReceived " + src + " " + length + ")");
    }

    public static void messageTransmitted(String name, int simtime, String dst, int length) {
	logBasic("(sensor " + name + " " + simtime +
		 " messageTransmitted " + dst + " " + length + ")");
    }

    public static void radarActivateSector(String name, int simtime, int sector) {
	logBasic("(sensor " + name + " " + simtime +
		 " radarActivateSector " + sector + ")");
    }

    public static void radarDeactivateSector(String name, int simtime, int sector) {
	logBasic("(sensor " + name + " " + simtime +
		 " radarDeactivateSector " + sector + ")");
    }

    public static void radarSetCurrentSector(String name, int simtime, int sector) {
	logBasic("(sensor " + name + " " + simtime +
		 " radarSetCurrentSector " + sector + ")");
    }

    public static void radarSetDataCollectionMode(String name, int simtime, int mode) {
	logBasic("(sensor " + name + " " + simtime +
		 " radarSetDataCollectionMode " + mode + ")");
    }

    public static void radarBeginScanning(String name, int simtime) {
	logBasic("(sensor " + name + " " + simtime + " radarBeginScanning)");
    }

    public static void radarEndScanning(String name, int simtime) {
	logBasic("(sensor " + name + " " + simtime + " radarEndScanning)");
    }

    public static void radarSetMeasurementDuration(String name, int simtime, int duration) {
	logBasic("(sensor " + name + " " + simtime +
		 " radarSetMeasurementDuration " + duration + ")");
    }

    public static void radarSetSampleSize(String name, int simtime, int size) {
	logBasic("(sensor " + name + " " + simtime +
		 " radarSetSampleSize " + size + ")");
    }

    public static void radarSetGain(String name, int simtime, int gain) {
	logBasic("(sensor " + name + " " + simtime +
		 " radarSetGain " + gain + ")");
    }


    // Target information
    private static HashMap targetStates = new HashMap();


    // Logging target state.  In basic mode, we only log when the
    // direction changes (but we log everything).  In verbose mode,
    // we log the target location every time, but direction and 
    // speed are only logged when the direction changes.

    public static void defTarget(String name, int simtime, 
				 double x, double y,
				 double direction, double speed) {
	
	targetStates.put(name, new Double(direction));
	logBasic("(deftarget " + name + " " + simtime +
		 "\n    location " + x + " " + y + " 0.0" +
		 "\n    direction " + direction +
		 "\n    speed " + speed + ")");
    }
	
    public static void targetState(String name, int simtime, 
				   double x, double y,
				   double direction, double speed) {

	// First get the old direction
	int oldDirection = -500;
	if ( targetStates.containsKey(name) ) {
	    oldDirection = ((Double) targetStates.get(name)).intValue();
	    targetStates.put(name, new Double(direction));
	}
	else
	    targetStates.put(name, new Double(direction));

	// Need to use ints because floats are too precise, and
	// end up recording too many minor changes.
	if (Math.abs(oldDirection - (int) Math.round(direction)) > 1) {
	    targetLocationBasic(name, simtime, x, y);
	    targetDirection(name, simtime, direction);
	    targetSpeed(name, simtime, speed);
	    targetStates.put(name, new Double(direction));
	}
	else 
	    targetLocationVerbose(name, simtime, x, y);
	
    }
    
    public static void targetLocationBasic(String name, int simtime, double x, double y) {
	logBasic("(target " + name + " " + simtime +
		 " location " + x + " " + y + " 0.0)");
    }

    public static void targetLocationVerbose(String name, int simtime, double x, double y) {
	logVerbose("(target " + name + " " + simtime +
		   " location " + x + " " + y + " 0.0)");
    }

    public static void targetDirection(String name, int simtime, double direction) {
	logBasic("(target " + name + " " + simtime +
		 " direction " + direction + ")");
    }

    public static void targetSpeed(String name, int simtime, double speed) {
	logBasic("(target " + name + " " + simtime +
		 " speed " + speed + ")");
    }
}
